"""this contains the server-side objects"""

from cubicweb import ValidationError
from cubicweb.selectors import implements
from cubicweb.server import hook

def check_wptitle(workcase, workpackage):
    """checks that `workpackage`'s title doesn't conflict with
    an existing workpackage in the same workcase
    """
    title = workpackage.title
    existing_wps = [wp.title for wp in workcase.split_into]
    if title in existing_wps:
        msg = 'There is already a workpackage named %s in this workcase'
        raise ValidationError(workpackage.eid,
                              {'title': workpackage._cw._(msg) % title})


class BeforeAddSplitIntoRelation(hook.Hook):
    """checks that the new workpackage's title doesn't conflict with
    an existing workpackage in the same workcase
    """
    __regid__ = 'checkwptitle_on_splitinto_change'
    __select__ = hook.Hook.__select__ & hook.match_rtype('split_into')
    events = ('before_add_relation',)

    def __call__(self):
        check_wptitle(self._cw.entity_from_eid(self.eidfrom),
                      self._cw.entity_from_eid(self.eidto))


class CheckWPName(hook.Hook):
    """checks that the new workpackage's title doesn't conflict with
    an existing workpackage in the same workcase
    """
    __regid__ = 'checkwptitle_on_wp_update'
    __select__ = hook.Hook.__select__ & implements('Workpackage')
    events = ('before_update_entity',)

    def __call__(self):
        if 'title' in self.entity.edited_attributes:
            # if the user hasn't updated/edited the title
            # it does not exist in the entity's dict-like interface
            check_wptitle(self.entity.reverse_split_into[0], self.entity)




# template's specific schema
from yams.buildobjs import EntityType, SubjectRelation, ObjectRelation, String, RichString
from cubicweb.schema import RQLConstraint, WorkflowableEntityType

class Workcase(WorkflowableEntityType):
    ref = String(required=True, fulltextindexed=True, indexed=True,
                 maxsize=16, unique=True)
    subject = String(fulltextindexed=True, maxsize=256)
    split_into = SubjectRelation('Workpackage', cardinality='*1',
                                 composite='subject')


class Workpackage(WorkflowableEntityType):
    title = String(required=True, fulltextindexed=True, indexed=True, maxsize=128)
    description = RichString(fulltextindexed=True)





"""this contains the template-specific entities' classes"""

from cubicweb.entities import AnyEntity, fetch_config

class Workcase(AnyEntity):
    __regid__ = 'Workcase'
    fetch_attrs, fetch_order = fetch_config(['ref', 'subject'])

    def dc_title(self):
        return self.ref.upper()

    def dc_long_title(self):
        return u'%s - %s' % (self.ref.upper(), self.subject)

    def dc_description(self, format='text/plain'):
        return self.printable_value('subject', format=format)


class WorkPackage(AnyEntity):
    __regid__ = 'Workpackage'
    fetch_attrs, fetch_order = fetch_config(['title'])

    def dc_title(self):
        return self.title

    def dc_long_title(self):
        ref = self.reverse_split_into[0].ref
        return u'%s - %s' % (ref, self.title)

    def parent(self):
        parents = self.reverse_split_into
        if parents:
            return parents[0]




rql('SET S name "pending" WHERE S name "waited", S state_of W, W name "workcase workflow"')
rql('SET S allowed_transition T WHERE S name "opened", S state_of W, W name "workcase workflow", '
    'T transition_of W, T name "failed"')
rql('SET S allowed_transition T WHERE S name "action", S state_of W, W name "workcase workflow", '
    'T transition_of W, T name "wait"')
rql('SET S allowed_transition T WHERE S name "action", S state_of W, W name "workcase workflow", '
    'T transition_of W, T name "done"')
rql('SET S allowed_transition T WHERE S name "pending", S state_of W, W name "workcase workflow", '
    'T transition_of W, T name "done"')
checkpoint()



"""template-specific forms/views/actions/components"""

from logilab.mtconverter import html_escape

from cubicweb.selectors import implements
from cubicweb.view import EntityView
from cubicweb.web import uicfg

uicfg.actionbox_appearsin_addmenu.tag_subject_of(('Workcase', 'split_into', 'Workpackage'), True)
uicfg.actionbox_appearsin_addmenu.tag_object_of(('Workcase', 'split_into', 'Workpackage'), False)




# -*- coding: utf-8 -*-
# postcreate script. You could setup a workflow here for example
wf = add_workflow('workpackage workflow', 'Workpackage')
reserve = wf.add_state(_(u'not started'), initial=True)
encours = wf.add_state(_(u'in progress'))
attente = wf.add_state(_(u'client validation'))
recette = wf.add_state(_(u'validated'))
garantie = wf.add_state(_(u'warranty'))

wf.add_transition(_(u'start'), reserve, encours)
wf.add_transition(_(u'done'), encours, attente)
wf.add_transition(_(u'warrant'), attente, garantie)
wf.add_transition(_(u'validate'), garantie, recette)

wf = add_workflow('workcase workflow', 'Workcase')
ouvert  = wf.add_state(_(u'opened'), initial=True)
action  = wf.add_state(_('action'))
attente = wf.add_state(_('pending'))
succes = wf.add_state(_(u'success'))
echec = wf.add_state(_(u'failure'))

wf.add_transition(_(u'open'), (action, attente), ouvert)
wf.add_transition(_(u'action'), (attente, ouvert), action)
wf.add_transition(_(u'wait'), (ouvert, action), attente)
wf.add_transition(_(u'failed'), (ouvert, action, attente), echec)
wf.add_transition(_(u'done'), (ouvert, action, attente), succes)




# pylint: disable-msg=W0622
"""cubicweb-workcase application packaging information"""

modname = 'workcase'
distname = "cubicweb-%s" % modname

numversion = (0, 6, 0)
version = '.'.join(str(num) for num in numversion)

license = 'LGPL'
copyright = '''Copyright (c) 2008-2010 LOGILAB S.A. (Paris, FRANCE).
http://www.logilab.fr/ -- mailto:contact@logilab.fr'''

author = "Logilab"
author_email = "contact@logilab.fr"

short_desc = "workcase component for the CubicWeb framework"
long_desc = """This CubicWeb component models workcase and workpackages
(a workcase can be split into several workpackages).

CubicWeb is a semantic web framework, see http://www.cubicweb.org
"""

from os import listdir as _listdir
from os.path import join, isdir

from glob import glob
scripts = glob(join('bin', 'workcase-*'))

web = 'http://www.cubicweb.org/project/%s' % distname

pyversions = ['2.4']

orig_dir = 'schemas'

#from cubicweb.devtools.pkginfo import get_distutils_datafiles
CUBES_DIR = join('share', 'cubicweb', 'cubes')
THIS_CUBE_DIR = join(CUBES_DIR, 'workcase')

def listdir(dirpath):
    return [join(dirpath, fname) for fname in _listdir(dirpath)
            if fname[0] != '.' and not fname.endswith('.pyc')
            and not fname.endswith('~')]

try:
    # data_files = get_distutils_datafiles('workcase')
    data_files = [
        # common files
        [THIS_CUBE_DIR, [fname for fname in glob('*.py') if fname != 'setup.py']],

        # client (web) files
        [join(THIS_CUBE_DIR, 'data'),  listdir('data')],
        [join(THIS_CUBE_DIR, 'i18n'),  listdir('i18n')],
        # Note: here, you'll need to add views' subdirectories if you want
        # them to be included in the debian package
        # server files
        [join(THIS_CUBE_DIR, 'migration'), listdir('migration')],
        ]
except OSError:
    # we are in an installed directory
    pass

__depends_cubes__ = {}
__depends__ = {'cubicweb': '>= 3.6.0'}
__use__ = tuple(__depends_cubes__)
classifiers = [
           'Environment :: Web Environment',
           'Framework :: CubicWeb',
           'Programming Language :: Python',
           'Programming Language :: JavaScript',
           ]
